@@ -12,7 +12,7 @@ from inspect import getframeinfo, stack
 from pathlib import Path, PurePath
 from stat import S_IWGRP, S_IWOTH, S_IWUSR, filemode
 from types import TracebackType
-from typing import TYPE_CHECKING, Any, Callable, Iterator, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Callable, Union
 from uuid import uuid4
 from weakref import WeakValueDictionary
 
@@ -21,6 +21,8 @@ import pytest
 from filelock import BaseFileLock, FileLock, SoftFileLock, Timeout, UnixFileLock, WindowsFileLock
 
 if TYPE_CHECKING:
+    from collections.abc import Iterator
+
     from pytest_mock import MockerFixture
 
 
@@ -63,7 +65,7 @@ def make_ro(path: Path) -> Iterator[None]:
     path.chmod(path.stat().st_mode | write)
 
 
-@pytest.fixture()
+@pytest.fixture
 def tmp_path_ro(tmp_path: Path) -> Iterator[Path]:
     with make_ro(tmp_path):
         yield tmp_path
@@ -81,7 +83,7 @@ def test_ro_folder(lock_type: type[BaseFileLock], tmp_path_ro: Path) -> None:
         lock.acquire()
 
 
-@pytest.fixture()
+@pytest.fixture
 def tmp_file_ro(tmp_path: Path) -> Iterator[Path]:
     filename = tmp_path / "a"
     filename.write_text("")
@@ -201,7 +203,24 @@ def test_nested_forced_release(lock_type: type[BaseFileLock], tmp_path: Path) ->
     assert not lock.is_locked
 
 
-_ExcInfoType = Union[Tuple[Type[BaseException], BaseException, TracebackType], Tuple[None, None, None]]
+@pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
+def test_nested_contruct(lock_type: type[BaseFileLock], tmp_path: Path) -> None:
+    # lock is re-entrant for a given file even if it is constructed multiple times
+    lock_path = tmp_path / "a"
+
+    with lock_type(str(lock_path), is_singleton=True, timeout=2) as lock_1:
+        assert lock_1.is_locked
+
+        with lock_type(str(lock_path), is_singleton=True, timeout=2) as lock_2:
+            assert lock_2 is lock_1
+            assert lock_2.is_locked
+
+        assert lock_1.is_locked
+
+    assert not lock_1.is_locked
+
+
+_ExcInfoType = Union[tuple[type[BaseException], BaseException, TracebackType], tuple[None, None, None]]
 
 
 class ExThread(threading.Thread):
@@ -287,7 +306,7 @@ def test_timeout(lock_type: type[BaseFileLock], tmp_path: Path) -> None:
     assert not lock_2.is_locked
 
     # try to acquire lock 2
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."):
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."):
         lock_2.acquire(timeout=0.1)
     assert not lock_2.is_locked
     assert lock_1.is_locked
@@ -316,44 +335,44 @@ def test_non_blocking(lock_type: type[BaseFileLock], tmp_path: Path) -> None:
     assert not lock_5.is_locked
 
     # try to acquire lock 2
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."):
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."):
         lock_2.acquire(blocking=False)
     assert not lock_2.is_locked
     assert lock_1.is_locked
 
     # try to acquire pre-parametrized `blocking=False` lock 3 with `acquire`
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."):
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."):
         lock_3.acquire()
     assert not lock_3.is_locked
     assert lock_1.is_locked
 
     # try to acquire pre-parametrized `blocking=False` lock 3 with context manager
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."), lock_3:
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."), lock_3:
         pass
     assert not lock_3.is_locked
     assert lock_1.is_locked
 
     # try to acquire pre-parametrized `timeout=0` lock 4 with `acquire`
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."):
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."):
         lock_4.acquire()
     assert not lock_4.is_locked
     assert lock_1.is_locked
 
     # try to acquire pre-parametrized `timeout=0` lock 4 with context manager
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."), lock_4:
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."), lock_4:
         pass
     assert not lock_4.is_locked
     assert lock_1.is_locked
 
     # blocking precedence over timeout
     # try to acquire pre-parametrized `timeout=-1,blocking=False` lock 5 with `acquire`
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."):
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."):
         lock_5.acquire()
     assert not lock_5.is_locked
     assert lock_1.is_locked
 
     # try to acquire pre-parametrized `timeout=-1,blocking=False` lock 5 with context manager
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."), lock_5:
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."), lock_5:
         pass
     assert not lock_5.is_locked
     assert lock_1.is_locked
@@ -380,7 +399,7 @@ def test_default_timeout(lock_type: type[BaseFileLock], tmp_path: Path) -> None:
     assert not lock_2.is_locked
 
     # try to acquire lock 2
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."):
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."):
         lock_2.acquire()
     assert not lock_2.is_locked
     assert lock_1.is_locked
@@ -388,7 +407,7 @@ def test_default_timeout(lock_type: type[BaseFileLock], tmp_path: Path) -> None:
     lock_2.timeout = 0
     assert lock_2.timeout == 0
 
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."):
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."):
         lock_2.acquire()
     assert not lock_2.is_locked
     assert lock_1.is_locked
@@ -442,7 +461,7 @@ def test_del(lock_type: type[BaseFileLock], tmp_path: Path) -> None:
     assert not lock_2.is_locked
 
     # try to acquire lock 2
-    with pytest.raises(Timeout, match="The file lock '.*' could not be acquired."):
+    with pytest.raises(Timeout, match=r"The file lock '.*' could not be acquired."):
         lock_2.acquire(timeout=0.1)
 
     # delete lock 1 and try to acquire lock 2 again
@@ -597,8 +616,8 @@ def test_soft_errors(tmp_path: Path, mocker: MockerFixture) -> None:
 def _check_file_read_write(txt_file: Path) -> None:
     for _ in range(3):
         uuid = str(uuid4())
-        txt_file.write_text(uuid)
-        assert txt_file.read_text() == uuid
+        txt_file.write_text(uuid, encoding="utf-8")
+        assert txt_file.read_text(encoding="utf-8") == uuid
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
@@ -663,31 +682,34 @@ def test_lock_can_be_non_thread_local(
 
 def test_subclass_compatibility(tmp_path: Path) -> None:
     class MyFileLock(FileLock):
-        def __init__(  # noqa: PLR0913 Too many arguments to function call (6 > 5)
+        def __init__(
             self,
             lock_file: str | os.PathLike[str],
             timeout: float = -1,
             mode: int = 0o644,
             thread_local: bool = True,
             my_param: int = 0,
-            **kwargs: dict[str, Any],
+            **kwargs: dict[str, Any],  # noqa: ARG002
         ) -> None:
-            pass
+            super().__init__(lock_file, timeout, mode, thread_local, is_singleton=True)
+            self.blocking = True
+            self.my_param = my_param
 
     lock_path = tmp_path / "a"
     MyFileLock(str(lock_path), my_param=1)
 
     class MySoftFileLock(SoftFileLock):
-        def __init__(  # noqa: PLR0913 Too many arguments to function call (6 > 5)
+        def __init__(
             self,
             lock_file: str | os.PathLike[str],
             timeout: float = -1,
             mode: int = 0o644,
             thread_local: bool = True,
             my_param: int = 0,
-            **kwargs: dict[str, Any],
+            **kwargs: dict[str, Any],  # noqa: ARG002
         ) -> None:
-            pass
+            super().__init__(lock_file, timeout, mode, thread_local, blocking=True, is_singleton=True)
+            self.my_param = my_param
 
     MySoftFileLock(str(lock_path), my_param=1)
 
@@ -725,12 +747,19 @@ def test_singleton_locks_are_distinct_per_lock_file(lock_type: type[BaseFileLock
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
 def test_singleton_locks_must_be_initialized_with_the_same_args(lock_type: type[BaseFileLock], tmp_path: Path) -> None:
     lock_path = tmp_path / "a"
-    lock = lock_type(str(lock_path), is_singleton=True)  # noqa: F841
+    args: dict[str, Any] = {"timeout": -1, "mode": 0o644, "thread_local": True, "blocking": True}
+    alternate_args: dict[str, Any] = {"timeout": 10, "mode": 0, "thread_local": False, "blocking": False}
 
-    with pytest.raises(ValueError, match="Singleton lock instances cannot be initialized with differing arguments"):
-        lock_type(str(lock_path), timeout=10, is_singleton=True)
-    with pytest.raises(ValueError, match="Singleton lock instances cannot be initialized with differing arguments"):
-        lock_type(str(lock_path), mode=0, is_singleton=True)
+    lock = lock_type(str(lock_path), is_singleton=True, **args)
+
+    for arg_name in args:
+        general_msg = "Singleton lock instances cannot be initialized with differing arguments"
+        altered_args = args.copy()
+        altered_args[arg_name] = alternate_args[arg_name]
+        with pytest.raises(ValueError, match=general_msg) as exc_info:
+            lock_type(str(lock_path), is_singleton=True, **altered_args)
+        exc_info.match(arg_name)  # ensure specific non-matching argument is included in exception text
+    del lock, exc_info
 
 
 @pytest.mark.skipif(hasattr(sys, "pypy_version_info"), reason="del() does not trigger GC in PyPy")
@@ -759,3 +788,51 @@ def test_singleton_instance_tracking_is_unique_per_subclass(lock_type: type[Base
     assert isinstance(Lock1._instances, WeakValueDictionary)  # noqa: SLF001
     assert isinstance(Lock2._instances, WeakValueDictionary)  # noqa: SLF001
     assert Lock1._instances is not Lock2._instances  # noqa: SLF001
+
+
+def test_singleton_locks_when_inheriting_init_is_called_once(tmp_path: Path) -> None:
+    init_calls = 0
+
+    class MyFileLock(FileLock):
+        def __init__(self, *args: Any, **kwargs: Any) -> None:  # noqa: ANN401
+            super().__init__(*args, **kwargs)
+            nonlocal init_calls
+            init_calls += 1
+
+    lock_path = tmp_path / "a"
+    lock1 = MyFileLock(str(lock_path), is_singleton=True)
+    lock2 = MyFileLock(str(lock_path), is_singleton=True)
+
+    assert lock1 is lock2
+    assert init_calls == 1
+
+
+def test_file_lock_positional_argument(tmp_path: Path) -> None:
+    class FilePathLock(FileLock):
+        def __init__(self, file_path: str) -> None:
+            super().__init__(file_path + ".lock")
+
+    lock_path = tmp_path / "a"
+    lock = FilePathLock(str(lock_path))
+    assert lock.lock_file == str(lock_path) + ".lock"
+
+
+@pytest.mark.parametrize(
+    ("lock_type", "expected_exc"),
+    [
+        (SoftFileLock, TimeoutError),
+        (FileLock, TimeoutError) if sys.platform == "win32" else (FileLock, PermissionError),
+    ],
+)
+def test_mtime_zero_exit_branch(
+    lock_type: type[BaseFileLock], expected_exc: type[BaseException], tmp_path: Path
+) -> None:
+    p = tmp_path / "z.lock"
+    p.touch()
+    Path(p).chmod(0o444)
+    os.utime(p, (0, 0))
+
+    lock = lock_type(str(p))
+
+    with pytest.raises(expected_exc):
+        lock.acquire(timeout=0)
